package com.sample.wui.jbpm;

import java.util.List;

import org.jbpm.task.Content;
import org.jbpm.task.Deadline;
import org.jbpm.task.EmailNotification;
import org.jbpm.task.Escalation;
import org.jbpm.task.Notification;
import org.jbpm.task.NotificationType;
import org.jbpm.task.OrganizationalEntity;
import org.jbpm.task.Reassignment;
import org.jbpm.task.Status;
import org.jbpm.task.Task;
import org.jbpm.task.TaskData;
import org.jbpm.task.service.EscalatedDeadlineHandler;
import org.jbpm.task.service.TaskService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MyEscalatedDeadlineHandler implements EscalatedDeadlineHandler {

	private static final Logger log = LoggerFactory
			.getLogger(MyEscalatedDeadlineHandler.class);

	private static MyEscalatedDeadlineHandler instance;

	public static MyEscalatedDeadlineHandler getInstance() {
		if (instance == null) {
			instance = new MyEscalatedDeadlineHandler();
		}
		return instance;
	}

	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.jbpm.task.service.EscalatedDeadlineHandler#executeEscalatedDeadline
	 * (org.jbpm.task.Task, org.jbpm.task.Deadline, org.jbpm.task.Content,
	 * org.jbpm.task.service.TaskService)
	 */

	private MyEscalatedDeadlineHandler() {
	}


	public void executeEscalatedDeadline(Task task, Deadline deadline,
			Content content, TaskService taskService) {

		if (deadline == null || deadline.getEscalations() == null) {
			return;
		}

		for (Escalation escalation : deadline.getEscalations()) {
			for (Notification notification : escalation.getNotifications()) {
				if (notification.getNotificationType() == NotificationType.Email) {
					executeEmailNotification((EmailNotification) notification,
							task);
				}
			}

			if (!escalation.getReassignments().isEmpty()) {
				// get first and ignore the rest.
				Reassignment reassignment = escalation.getReassignments()
						.get(0);
				task.getTaskData().setStatus(Status.Ready);
				List<OrganizationalEntity> potentialOwners = reassignment
						.getPotentialOwners();
				task.getPeopleAssignments().setPotentialOwners(potentialOwners);
			}
		}
		deadline.setEscalated(true);
	}

	/**
	 * Execute email notification.
	 * 
	 * @param notification
	 *            the notification
	 * @param task
	 *            the task
	 */
	private void executeEmailNotification(EmailNotification notification,
			Task task) {
		// notificationService = (NotificationService)
		// lookup(NOTIFICATION_SERVICE_JNDI);
		TaskData taskData = task.getTaskData();
		Long processInstanceId = taskData.getProcessInstanceId();
		Long taskId = task.getId();
		// String userName = taskData.getActualOwner().getId();
		String processId = taskData.getProcessId();

		// notificationService.notifyChange(userName, processId,
		// processInstanceId, taskId);
		log.info("Notification data.. processInstanceId: " + processInstanceId
				+ " taskId: " + taskId + "  processId: " + processId);

	}

	// /**
	// * Lookup.
	// *
	// * @param jndiName
	// * the jndi name
	// * @return the object
	// */
	// private Object lookup(String jndiName) {
	//
	// final Hashtable<String, Object> jndiProperties = new Hashtable<String,
	// Object>();
	// jndiProperties.put(Context.URL_PKG_PREFIXES,
	// "org.jboss.ejb.client.naming");
	//
	// jndiProperties.put("jboss.naming.client.ejb.context", true);
	// // jndiProperties.put(Context.INITIAL_CONTEXT_FACTORY,
	// // "org.jboss.naming.remote.client.InitialContextFactory");
	// jndiProperties.put(Context.PROVIDER_URL, "remote://localhost:4447");
	// try {
	// Context ctx = new InitialContext(jndiProperties);
	// return ctx.lookup(jndiName);
	// } catch (Exception e) {
	// throw new IllegalStateException("Problem with jndi lookup for "
	// + jndiName);
	// }
	//
	// }

}
